/* Copyright (c) 2009, Canonical Ltd. All rights reserved. */

YUI({
    base: '../../yui/current/build/',
    filter: 'raw',
    combine: false
    }).use('lazr.activator', 'lazr.testing.runner', 'node',
           'event', 'event-simulate', 'console', function(Y) {

var Assert = Y.Assert;  // For easy access to isTrue(), etc.

/*
 * A wrapper for the Y.Event.simulate() function.  The wrapper accepts
 * CSS selectors and Node instances instead of raw nodes.
 */
function simulate(selector, evtype) {
    var rawnode = Y.Node.getDOMNode(Y.get(selector));
    Y.Event.simulate(rawnode, evtype);
}

/* Helper function to clean up a dynamically added widget instance. */
function cleanup_widget(widget) {
    // Nuke the boundingBox, but only if we've touched the DOM.
    if (widget.get('rendered')) {
        var bb = widget.get('boundingBox');
        bb.get('parentNode').removeChild(bb);
    }
    // Kill the widget itself.
    widget.destroy();
}

var suite = new Y.Test.Suite("Activator Tests");


suite.add(new Y.Test.Case({

    name: 'activator_basics',

    setUp: function() {
        this.workspace = Y.get('#workspace');
        if (!this.workspace){
            Y.get(document.body).appendChild(Y.Node.create(
                '<div id="workspace" ' +
                'style="border: 1px solid blue; ' +
                'width: 20em; ' +
                'margin: 1em; ' +
                'padding: 1em">'+
                '</div>'));
            this.workspace = Y.get('#workspace');
        }
        this.workspace.appendChild(Y.Node.create(
            '<div id="example-1">' +
            '<span class="yui-activator-data-box">' +
            '    Original Value' +
            '</span>' +
            '<button ' +
            ' class="lazr-btn yui-activator-act yui-activator-hidden">' +
            '    Go' +
            '</button>' +
            '<div class="yui-activator-message-box yui-activator-hidden">' +
            '</div>' +
            '</div>'));
        this.activator = new Y.lazr.activator.Activator(
            {contentBox: Y.get('#example-1')});
        this.action_button = this.activator.get('contentBox').query(
            '.yui-activator-act');
    },

    tearDown: function() {
        cleanup_widget(this.activator);
        this.workspace.set('innerHTML', '');
    },

    test_unhiding_action_button: function() {
        this.action_button.addClass('yui-activator-hidden');
        Assert.isTrue(this.action_button.hasClass('yui-activator-hidden'));
        this.activator.render();
        Assert.isFalse(
            this.action_button.hasClass('yui-activator-hidden'),
            "yui-activator-hidden class wasn't removed from the " +
            "action button");
    },

    test_simulate_click_on_action_button: function() {
        this.activator.render();
        this.activator.subscribe('act', function(e) {
            this.resume();
        }, this);
        simulate(this.action_button, 'click');
        this.wait(function () {
            Assert.fail("'act' event wasn't fired.");
        }, 3000);
    },

    test_renderSuccess: function() {
        this.activator.render();
        var data = Y.Node.create('new value');
        var message = Y.Node.create('success message');
        Assert.isFalse(
            this.activator.get('contentBox').hasClass(
                'yui-activator-success'),
            'The widget is not setup propertly.');

        this.activator.renderSuccess(data, message);

        Assert.isTrue(
            this.activator.get('contentBox').hasClass(
                'yui-activator-success'),
            'renderSuccess did not add the success css class');

        var data_box = this.activator.get('contentBox').query(
            '.yui-activator-data-box');
        Assert.areEqual(
            'new value',
            data_box.get('innerHTML'),
            'renderSuccess did not set the contents of the data-box');

        var message_body = this.activator.get('contentBox').query(
            '.yui-activator-message-body');

        Assert.areEqual(
            'success message',
            message_body.get('innerHTML'),
            'renderSuccess did not set the contents of the message-body');
    },

    test_renderProcessing: function() {
        this.activator.render();
        var message = Y.Node.create('<b>processing message</b>');
        Assert.isFalse(
            this.activator.get('contentBox').hasClass(
                'yui-activator-processing'),
            'The widget is not setup propertly.');

        this.activator.renderProcessing(message);

        Assert.isTrue(
            this.activator.get('contentBox').hasClass(
                'yui-activator-processing'),
            'renderProcessing did not add the processing css class');

        var message_body = this.activator.get('contentBox').query(
            '.yui-activator-message-body');

        Assert.areEqual(
            '<b>processing message</b>',
            message_body.get('innerHTML'),
            'renderProcessing did not set the contents of the message-body');
    },

    test_renderCancellation: function() {
        this.activator.render();
        var message = Y.Node.create('<b>cancel message</b>');
        Assert.isFalse(
            this.activator.get('contentBox').hasClass(
                'yui-activator-cancellation'),
            'The widget is not setup propertly.');

        this.activator.renderCancellation(message);

        Assert.isTrue(
            this.activator.get('contentBox').hasClass(
                'yui-activator-cancellation'),
            'renderCancellation did not add the cancel css class');

        var message_body = this.activator.get('contentBox').query(
            '.yui-activator-message-body');
        Assert.areEqual(
            '<b>cancel message</b>',
            message_body.get('innerHTML'),
            "renderCancellation didn't set the contents of the message-body");
    },

    test_renderFailure: function() {
        this.activator.render();
        var message = Y.Node.create('failure message');
        Assert.isFalse(
            this.activator.get('contentBox').hasClass(
                'yui-activator-failure'),
            'The widget is not setup propertly.');

        this.activator.renderFailure(message);

        Assert.isTrue(
            this.activator.get('contentBox').hasClass(
                'yui-activator-failure'),
            'renderFailure did not add the failure css class');

        var message_body = this.activator.get('contentBox').query(
            '.yui-activator-message-body');

        Assert.areEqual(
            'failure message',
            message_body.get('innerHTML'),
            'renderFailure did not set the contents of the message-body');
    },

    test_empty_message_box: function() {
        // If no message_node is passed to renderFailure(),
        // the message box is hidden.
        this.activator.render();
        this.activator.renderFailure();

        var message_box = this.activator.get('contentBox').query(
            '.yui-activator-message-box');

        Assert.isTrue(
            message_box.hasClass('yui-activator-hidden'),
            "Message box should be hidden.");
        Assert.areEqual(
            '',
            message_box.get('innerHTML'),
            'Message box contents should be empty.');
    },

    test_closing_message_box: function() {
        this.activator.render();
        this.activator.renderFailure(Y.Node.create('short message'));

        var message_box = this.activator.get('contentBox').query(
            '.yui-activator-message-box');
        var message_body = this.activator.get('contentBox').query(
            '.yui-activator-message-body');
        var message_close_button = this.activator.get('contentBox').query(
            '.yui-activator-message-close');
        simulate(message_close_button, 'click');
        Assert.isTrue(
            message_box.hasClass('yui-activator-hidden'),
            "Message box should be hidden.");
        Assert.areEqual(
            'short message',
            message_body.get('innerHTML'),
            'Message body contents should still be there.');
    },

    test_widget_does_not_have_a_tabindex_when_focused: function() {
        // The tabindex attribute appears when the widget is focused.
        this.activator.render();
        this.activator.focus();

        Assert.isFalse(
            this.activator.get('boundingBox').hasAttribute('tabindex'),
            "The widget should not have a tabindex attribute.");
    }
}));

Y.lazr.testing.Runner.add(suite);
Y.lazr.testing.Runner.run();

});
