"""
A script to automatically create the files for a new widget.

Usage: $ scaffold.py <new-widget-name>
"""

import sys
import string
from os.path import join, exists, abspath, dirname, isdir
from os import mkdir


usage = """
A script to automatically create the files for a new widget.

Usage: %s <new-widget-name>
""" % sys.argv[0]

# The default location of our skeleton files.
default_skel_dir = join(dirname(abspath(__file__)), 'skel')


def find_path(root, path):
    """Set a template file path, raising an error if it could not be found.

    Calls sys.exit() if the file could not be found.

    TODO: Need to find a more graceful way to exit.
    """
    tpath = join(root, path)
    if not exists(tpath):
        print >>sys.stderr, "Failed to find file: %s" % tpath
        sys.exit(1)
    return tpath

def write_template(template, fpath, **kwds):
    """Write a file from a template, using kwargs for template variables."""
    tstr = string.Template(open(template).read())
    newfile = open(fpath, 'w')
    try:
        newfile.write(tstr.safe_substitute(kwds))
    finally:
        newfile.close()
    print "Created:  ", fpath

def make_path(*args):
    """Join the arguments into a path and create a new directory for them.

    Returns the new path.
    """
    root = ''
    for path in args:
        root = join(root, path)
        if not exists(root):
            mkdir(root)
            print "Created:  ", root
        elif not isdir(root):
            print >>sys.stderr, "Failed:'%s' exists and is not a directory" % root
            sys.exit(1)
    return root

def make_widget(name, skel_dir=None):
    """Make a new widget scaffold with the supplied name."""
    if skel_dir is None:
        skel_dir = default_skel_dir

    js_template = find_path(skel_dir, 'template.js')
    test_htm_template = find_path(skel_dir, 'template.test-html')
    test_js_template = find_path(skel_dir, 'template.test-js')
    example_template = find_path(skel_dir, 'template.example')
    css_template = find_path(skel_dir, 'template.css')
    src_dir = find_path('.', 'src-js')
    example_dir = find_path('.', 'examples')

    def write_file(template, dest):
        # A quick closure to fill in common template variables
        write_template(template, dest, WIDGET_NAME=name)

    # source files
    src = make_path(src_dir, 'lazrjs', name)
    write_file(js_template, join(src, name + '.js'))

    # assets
    assets = make_path(src, 'assets')
    write_file(css_template, join(assets, name + '-core.css'))

    # skins
    skins = make_path(assets, 'skins')
    sam = make_path(skins, 'sam')
    write_file(css_template, join(sam, 'skin.css'))

    # tests
    tests = make_path(src, 'tests')
    write_file(test_htm_template, join(tests, 'index.html'))
    write_file(test_js_template, join(tests, name + '.js'))

    # examples
    example = make_path(example_dir, name)
    write_file(example_template, join(example, 'index.html'))


def main():
    if len(sys.argv) != 2:
        print usage
        sys.exit(1)

    make_widget(sys.argv[1])
