import os
import mocker
import unittest
import pkg_resources
import xml.dom

from lazr.js.build import CSSComboFile


class TestCSSComboFile(CSSComboFile):
    """A special CSSComboFile that doesn't log to the stdout."""

    def log(self, msg):
        pass


class CSSComboFileTest(mocker.MockerTestCase):

    def test_update_removes_target_on_error(self):
        tempdir = self.makeDir()
        broken_css = self.makeFile(
            content=".broken { ---; }",
            suffix=".css", dirname=tempdir)
        target_css = os.path.join(tempdir, "target.css")
        css_combo = TestCSSComboFile([broken_css], target_css)
        self.assertRaises(xml.dom.SyntaxErr, css_combo.update)
        self.failIf(
            os.path.exists(target_css), "Target file shouldn't exist.")

    def test_update_logs_filename(self):
        tempdir = self.makeDir()
        sample_css = self.makeFile(
            content=".sample {}", basename="sample.css", dirname=tempdir)
        target_css = os.path.join(tempdir, "target.css")
        css_combo = TestCSSComboFile([sample_css], target_css)
        mock = self.mocker.patch(css_combo)
        mock.log("Processing 'sample.css'")
        self.mocker.replay()

        css_combo.update()

    def test_update_honors_ie_hacks(self):
        tempdir = self.makeDir()

        ie_css = self.makeFile(
            content="""
/* IE Hacks should be preserved */
html {
    *zoom: 1;
}
            """, basename='ie_hacks.css', dirname=tempdir)
        target_css = os.path.join(tempdir, "target.css")
        css_combo = TestCSSComboFile([ie_css], target_css)
        css_combo.update()

        target_fh = open(target_css, 'r')
        output = target_fh.read()
        target_fh.close()
        self.assertEquals(output, "/* ie_hacks.css */\nhtml{*zoom:1}\n")


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)
